<?php

require_once(MOD_PATH.'Dictionaries/TypesOfProcessesStates/beans/TypeOfProcessStateBean.inc');
require_once(MOD_PATH.'Dictionaries/TypesOfProcessesStates/TypeOfProcessesStatesService.inc');


/**
 * TypeOfProcessStateForm
 * @uses      DialogForm
 * @final
 * @author    Tomasz Świenty
 * @version   2.0
 * @copyright Copyright (c) BetaSoft
 */
final class TypeOfProcessStateForm extends DialogForm {

    /**
     * cMethods
     * @static
     * @var array
     * @access protected
     */
    protected static $cMethods = [
        'delete',
        'loadTypeByClsnam',
    ];

    /**
     * @var int|string
     */
    private $HWND;

    /**
     * __construct
     *
     * @param string $name
     * @param string $caption
     * @param string $style
     *
     * @access public
     * @return void
     */
    public function __construct($name, $caption = NULL, $style = BS_DIALOG) {

        parent::__construct($name, Translator::translate('Definicja statusu'), $style);

        $this->path = Application::makeRelativePath(__FILE__);
        $this->HWND = $this->createHWND();

        $this->db = PgManager::getInstance();

        $this->width = '360px';
        $this->height = '440px';

        if (Request::hasMethodCall($this->HWND, array_keys(self::$cMethods))) {
            return;
        }

        if (($this->isOpened()) && (!$this->noNeedForCreate())) {
            $this->create();
        }

    }

    /**
     * getCallableMethods
     * @access public
     * @return array
     */
    public function getCallableMethods() {
        return self::$cMethods;
    }

    /**
     * delete
     *
     * @param string $params
     *
     * @access public
     * @return bool
     */
    public function delete($params) {

        $data = JSON::toArray($params);

        $bean = new TypeOfProcessStateBean($data['tpstid']);
        $bean->delete();

        $service = new TypeOfProcessesStatesService();
        $service->removeStampImage($bean);

        JScript::notifySuccess(Translator::translate('Definicja statusu została usunięta'));
        if ((isset($data['afterSubmit'])) && ($data['afterSubmit'])) {
            JScript::add(''.$data['afterSubmit'].';');
        }

        return TRUE;

    }

    /**
     * loadTypeByClsnam
     *
     * @param string $clsnam
     *
     * @return string
     */
    public function loadTypeByClsnam($clsnam, $toHTML = TRUE) {

        $sData = $this->getSData();
        $out = [];

        if ($clsnam == 'DOCUMENT') {
            require_once(MOD_PATH.'ADocuments/ADocumentsService.inc');
            ADocumentsService::loadDocTypes(TRUE);
            $docTypes = ADocumentsService::getDocTypes();
            if ((is_array($docTypes)) AND (!empty($docTypes))) {
                foreach ($docTypes as $dctpid => $docType) {
                    $out[$dctpid] = [
                        $dctpid,
                        $docType['dctpnm'],
                    ];
                }
            }

            if (isset($sData['type__']) && !isset($out[$sData['type__']])) {
                require_once(MOD_PATH.'Dictionaries/TypesOfDocuments/services/TypesOfDocumentsService.inc');
                $typesOfDocumentsService = new TypesOfDocumentsService();
                $typeOfDocument = $typesOfDocumentsService->getInstanceByDocTypeID($sData['type__']);

                if ($typeOfDocument) {
                    $out[$sData['type__']] = [
                        $sData['type__'],
                        $typeOfDocument->get('dctpnm'),
                    ];
                }
            }
        } elseif ($clsnam == 'PROCESS') {
            $res = $this->db->query('SELECT dos_id, COALESCE(dossmb || \' \' || ndenam, ndenam) FROM dossiers_list dl WHERE dl.ent_id = '.SysContext::$ent_id.' AND dl.is_del IS NOT TRUE', FALSE, PGSQL_NUM);
            if (is_array($res))
                $out += $res;
            $res = NULL;
        } elseif ($clsnam == 'DEVICE') {
            $res = $this->db->query('SELECT decaid, decanm FROM devices_category WHERE ent_id = '.SysContext::$ent_id.'', FALSE, PGSQL_NUM);
            if (is_array($res))
                $out += $res;
            $res = NULL;
        } elseif ($clsnam == 'CREGISTER') {
            $res = $this->db->query('SELECT id____, label_ FROM cregisters.register WHERE is_del IS FALSE AND cretpe = 1 ORDER BY label_', FALSE, PGSQL_NUM);
            if (is_array($res))
                $out += $res;
            $res = NULL;
        } elseif ($clsnam == 'EVENT') {
            require_once(MOD_PATH.'Tasks/TasksService.inc');

            if (isset(TasksService::$fontListMap2) && isset(TasksService::$fontListMap2)) {
                foreach (TasksService::$fontListMap2 as $trmtyp => $font) {
                    if (isset(TasksService::$listMap[$trmtyp]['caption'])) {
                        $out[] = [
                            $trmtyp,
                            TasksService::$listMap[$trmtyp]['caption'],
                        ];
                    }
                }
            }
        }

        require_once(LIB_PATH.'forms/ExtSelect.inc');
        $type__ = new ExtSelect($this->name.'type__', TRUE);
        $type__->addAssocArray($out, 0, 1);

        if ($out) {
            $type__->withDefaultItem = Translator::translate('wybierz');
        } else {
            $type__->withDefaultItem = Translator::translate('brak');
        }

        if ($toHTML) {
            return $type__->toHtml('');
        }

        return $out;

    }

    /**
     * @param null $params
     * @param null $data
     *
     * @throws UserRightsException
     */
    public function open($params = NULL, &$data = NULL) {

        if (!UserRights::checkSysAcc('bswfms.settings.level2')) {
            throw new UserRightsException(NULL, 'bswfms.settings.level2');
        }

        $this->clear();

        if (is_numeric($params)) {
            $data['tpstid'] = $params;
        } else {
            $data = JSON::toArray($params);
        }

        $tpstid = isset($data['tpstid']) ? $data['tpstid'] : NULL;

        if ($tpstid !== NULL) {
            $bean = new TypeOfProcessStateBean($tpstid);
            $dd = explode('|', $bean->data['clsnam']);
            $bean->data['clsnam'] = $dd[0];
            if (isset($dd[1])) {
                $bean->data['type__'] = $dd[1];
            }
        }

        if ((isset($data['mode'])) && ($tpstid) && ($data['mode'] == 'delete')) {
            $js = '$A1(\''.$this->HWND.'\',\''.$this->name.'\','.(int)array_search('delete', self::$cMethods).',({tpstid:'.$tpstid.',afterSubmit:\''.$data['afterSubmit'].'\'}).toJSONString());';
            JScript::confirmDelete(sprintf(Translator::translate('Czy na pewno chcesz usunąć wybrany status %s?'), $bean->data['dscrpt']), $js);
            return;
        }

        parent::open($params, $data);
        $this->create();

        if ($tpstid) {
            $data = array_merge($data, $bean->data);
            $this->setSData($data);

            $dd = $this->loadTypeByClsnam($data['clsnam'], FALSE);
            if ($dd) {
                $this->type__->addAssocArray($dd, 0, 1);
            }

            $this->caption .= ' - '.$bean->dscrpt;
        }

        $this->setWData($data);

    }

    /**
     * @return bool
     * @throws RequestAbortException
     * @throws UserRightsException
     * @throws WidgetException
     */
    public function save() {
        if (!$this->isOpened()) {
            return FALSE;
        }

        if (!UserRights::checkSysAcc('bswfms.settings.level2')) {
            throw new UserRightsException(NULL, 'bswfms.settings.level2');
        }

        $sdata = $this->getSData();
        $data = $this->getData();

        if (!$data['dscrpt']) {
            throw new WidgetException($this->name.'dscrpt', Translator::translate('Wypełnij pole.'));
        }

        if (!is_numeric($data['state_'])) {
            throw new WidgetException($this->name.'state_', Translator::translate('Niepoprawne dane. Należy wpisać liczbę całkowitą.'));
        }

        if ($data['type__']) {
            $data['clsnam'] = $data['clsnam'].'|'.$data['type__'];
        }

        $tpstid = isset($data['tpstid']) ? $data['tpstid'] : NULL;

        if ($data['status'] === 'INITIAL') {
            $q = $this->db->query('SELECT tpstid FROM types_of_processes_states WHERE status = \''.$data['status'].'\' '.($tpstid ? ' AND tpstid <> '.$tpstid.' ' : NULL).' AND clsnam = \''.$data['clsnam'].'\'', FALSE, PGSQL_ASSOC);
            if (isset($q[0]['tpstid'])) {
                throw new WidgetException($this->name.'status', sprintf(Translator::translate('Status %s klasy %s już instnieje.'), $data['status'], $data['clsnam']));
            }
        }

        $q = $this->db->query('SELECT tpstid FROM types_of_processes_states WHERE state_ = \''.$data['state_'].'\' '.($tpstid ? ' AND tpstid <> '.$tpstid.' ' : NULL).' AND clsnam = \''.$data['clsnam'].'\'', FALSE, PGSQL_ASSOC);
        if (isset($q[0]['tpstid'])) {
            throw new WidgetException($this->name.'state_', sprintf(Translator::translate('Kod statusu %s klasy %s już istnieje. Wprowadź inny kod.'), $data['state_'], $data['clsnam']));
        }

        $bean = new TypeOfProcessStateBean($tpstid);
        $bean->set('state_', $data['state_']);
        $bean->set('dscrpt', $data['dscrpt']);
        $bean->set('status', $data['status']);
        $bean->set('colour', $data['colour']);
        $bean->set('clsnam', $data['clsnam']);
        $bean->set('is_hid', $data['is_hid']);

        $doSave = TRUE;
        $msg = sprintf(Translator::translate('Modyfikacja pola GUID może spowodować nieprawidłowe działanie aplikacji!</br></br>Kontynuować zapis?'));
        if ($tpstid !== NULL && $sdata['tpguid'] !== $data['tpguid'] && !(bs_empty_str($sdata['tpguid']) && bs_empty_str($data['tpguid']))) {
            $doSave = FALSE;
            if (ConfirmBox::sweetConfirm($this->HWND, [
                    'html' => $msg,
                    'type' => 'warning',
                ]) == ConfirmBox::CONFIRM_YES) {
                $doSave = TRUE;
            }
        }

        if ($tpstid === FALSE || $doSave) {
            $bean->set('tpguid', $data['tpguid']);
            $bean->save();

            $service = new TypeOfProcessesStatesService();
            $service->createStampImage($bean);

            if ($this->afterSave) {
                Application::execMsg($this->afterSave, $bean->getPKeyValue());
            }

            $this->jsClose(TRUE, FALSE);
        }

        return TRUE;
    }

    /**
     * create
     * @access public
     * @return void
     */
    private function create() {

        if (!$this->isOpened()) {
            return FALSE;
        }

        $this->dscrpt = new TextInput2($this->name.'dscrpt');
        $this->dscrpt->width = 'calc(100% - '.(2 * Dialog::SPACE_LEFT).'px)';
        $this->dscrpt->setLabel(Translator::translate('Opis').':');
        $this->dscrpt->setRequired();

        $clsnam = new ExtSelect($this->name.'clsnam');
        $clsnam->addItem(NULL, '--'.Translator::translate('wybierz').'--');
        $clsnam->addItem('PROCESS', Translator::translate('Sprawa'));
        $clsnam->addItem('DOCUMENT', Translator::translate('Dokument'));
        $clsnam->addItem('DEVICE', Translator::translate('Urządzenie'));
        $clsnam->addItem('CAMPAIGN', Translator::translate('Kampania'));
        $clsnam->addItem('CREGISTER', Translator::translate('Rejestry'));
        //$clsnam->addItem('DOCUMENT|VATNOTE',Translator::translate('Dokument / Faktura'));
        //$clsnam->addItem('DOCUMENT|PAPER',Translator::translate('Dokument / Pismo'));
        $clsnam->addItem('RCP', Translator::translate('Karta RCP'));
        $clsnam->addItem('FKDEMANDELEMENT', Translator::translate('Element listy zapotrzebowania'));
        $clsnam->addItem('FKORDERINELEMENT', Translator::translate('Element listy zamówienia obcego'));
        $clsnam->addItem('FKORDEROUTELEMENT', Translator::translate('Element listy zamówienia własnego'));
        $clsnam->addItem('CONTACT', Translator::translate('Klient'));
        $clsnam->addItem('EVENT', Translator::translate('Zdarzenie'));
        $clsnam->onChange = 'App.'.$this->name.'loadTypeByClsnam(this.value);';
        $clsnam->setLabel(Translator::translate('Klasa').':');
        $clsnam->setRequired();

        $this->type__ = new ExtSelect($this->name.'type__', TRUE);
        $ltype__ = $this->type__->setLabel(Translator::translate('Podtyp').':');
        $ltype__->helpText = Translator::translate('Podtyp klasy wybranej powyżej. Np. dla dokumentów jest to konkrenty typ dokumentu. W przypadku nie wybrania podtypu na formularzu dokumentu zostaną załadowane wszystkie zdefiniowane statusu, w których nie określono podtypu. <br><br><b>Pole zostanie załadowane dostępnymi podtypami w momencie wybrania klasy.</b>');

        $status = new ExtSelect($this->name.'status');
        $status->addItem('INITIAL', 'INITIAL - '.Translator::translate('początkowy'));
        $status->addItem('TEMPORARY', 'TEMPORARY - '.Translator::translate('przejściowy'));
        $status->addItem('ACCEPTED', 'ACCEPTED - '.Translator::translate('zaakceptowany'));
        $status->addItem('FINAL', 'FINAL - '.Translator::translate('końcowy'));
        $status->addItem('CANCEL', 'CANCEL - '.Translator::translate('anulowany'));
        $status->setLabel(Translator::translate('Typ statusu'));
        $status->setRequired();
        $status->width = 'calc(60% - 15px)';

        $state_ = new NumberInput2($this->name.'state_');
        $state_->width = '100px';
        $state_->numberType = 'positive_int';
        $lstate = $state_->setLabel(Translator::translate('Kod statusu').':');
        $lstate->helpText = Translator::translate('Kod statusu przechowywany w innych tabelach. Należy wpisać liczbę całkowitą.');
        $state_->setRequired();

        $colour = new ColorPickerEx($this->name.'urPrev');
        $colour->width = '100px';
        $colour->class = 'ModernFlat';
        $lcolour = $colour->setLabel(Translator::translate('Kolor').':');
        $lcolour->helpText = Translator::translate('Aby wybrać kolor kliknij na pole obok.');

        $is_hid = new CheckBox2($this->name.'is_hid', FALSE, TRUE);
        $is_hid->leftCaption = Translator::translate('ukryty');
        $is_hid->helpText = Translator::translateJS('Status oznaczony tym atrybutem podlega specjalnemu traktowaniu. <br><br> Nie ma możliwości ręcznego wskazania takiego statusu z komponentu do wyboru statusów (chyba że jest już ustawiony).<br><br> Status taki można ustawić tylko poprzez <b>procedurę</b> lub za pomocą mechnizmu <b>custom widget</b>.');

        $tpguid = new TextInput2($this->name.'tpguid');
        $tpguid->setLabel(Translator::translate('Guid'));

        $composer = new DialogComposer($this);
        $composer->addNext($this->dscrpt, 'dscrpt');
        $composer->addNext($clsnam, 'clsnam');
        $composer->addNext($this->type__, 'type__');
        $composer->addNext($status, 'status');
        $composer->addAfter($status, $state_, 'state_', Widget::P_TO_RIGHT);
        $composer->addAfter($state_, $colour, 'colour');
        $composer->addAfter($status, $tpguid, 'tpguid');
        $composer->addAfter($tpguid, $is_hid, 'is_hid');

        $composer->addModernButtonsPanel([
            'bSave',
            'bClose',
        ]);
        $this->add($composer);

        JScript::add('App.'.$this->name.'loadTypeByClsnam = function(clsnam) {
            $A1(\''.$this->HWND.'\', \''.$this->name.'\', '.(int)array_search('loadTypeByClsnam', self::$cMethods).', clsnam, \'if (text) {$(\\\''.$this->type__->name.ExtSelect::SELECT_NAME_POSTFIX.'\\\').innerHTML = text;}\', 0, true);
        }');

    }

} // class TypeOfProcessStateForm